@@ -1,7 +1,7 @@
 /***************************************************************************
  $RCSfile$
                              -------------------
-    cvs         : $Id: transfinder.cpp 410 2007-09-04 07:07:26Z martin $
+    cvs         : $Id$
     begin       : Mon Mar 01 2004
     copyright   : (C) 2004 by Martin Preuss
     email       : martin@libchipcard.de
@@ -164,144 +164,148 @@ TransactionFinder::~TransactionFinder(){
 
 
 bool TransactionFinder::init(){
-  GWEN_DB_NODE *db;
+  GWEN_DB_NODE *dbConfig=NULL;
   GWEN_DB_NODE *dbT;
   std::string s;
   int i, j, idx;
+  int rv;
 
-  categoryListView->beginModify();
-  db=_app->getAppData();
-  if (db) {
-    db=GWEN_DB_GetGroup(db, GWEN_PATH_FLAGS_NAMEMUSTEXIST,
-                        "gui/dlgs/transactionFinder");
-    if (db) {
-      GWEN_DB_NODE *dbT;
-      int w, h;
-
-      w=GWEN_DB_GetIntValue(db, "width", 0, 640);
-      h=GWEN_DB_GetIntValue(db, "height", 0, 480);
-      resize(w, h);
-
-      dbT=GWEN_DB_GetGroup(db, GWEN_PATH_FLAGS_NAMEMUSTEXIST, "accountList");
-      if (dbT) {
-        const char *p;
-
-        /* found settings */
-        i=GWEN_DB_GetIntValue(dbT, "sortColumn", 0, -1);
-        if (i!=-1)
-          accountListView->setSortColumn(i);
-        p=GWEN_DB_GetCharValue(dbT, "sortOrder", 0, "ascending");
-        if (p) {
-          if (strcasecmp(p, "ascending")==0)
-            accountListView->setSortOrder(Qt::Ascending);
-          else
-            if (strcasecmp(p, "descending")==0)
-              accountListView->setSortOrder(Qt::Descending);
-        }
-        for (i=0; i<accountListView->columns(); i++) {
-          accountListView->setColumnWidthMode(i, QListView::Manual);
-          j=GWEN_DB_GetIntValue(dbT, "columns", i, -1);
-          if (j!=-1)
-          accountListView->setColumnWidth(i, j);
-        } /* for */
-      } // if account list data
-
-      dbT=GWEN_DB_GetGroup(db, GWEN_PATH_FLAGS_NAMEMUSTEXIST, "categoryList");
-      if (dbT) {
-        const char *p;
-
-        /* found settings */
-        i=GWEN_DB_GetIntValue(dbT, "sortColumn", 0, -1);
-        if (i!=-1)
-          categoryListView->setSortColumn(i);
-        p=GWEN_DB_GetCharValue(dbT, "sortOrder", 0, "ascending");
-        if (p) {
-          if (strcasecmp(p, "ascending")==0)
-            categoryListView->setSortOrder(Qt::Ascending);
-          else
-            if (strcasecmp(p, "descending")==0)
-              categoryListView->setSortOrder(Qt::Descending);
-        }
-        for (i=0; i<categoryListView->columns(); i++) {
-          categoryListView->setColumnWidthMode(i, QListView::Manual);
-          j=GWEN_DB_GetIntValue(dbT, "columns", i, -1);
-          if (j!=-1)
-          categoryListView->setColumnWidth(i, j);
-        } /* for */
-      } // if category list data
-
-      dbT=GWEN_DB_GetGroup(db, GWEN_PATH_FLAGS_NAMEMUSTEXIST,
-                           "transactionList");
-      if (dbT) {
-        const char *p;
-
-        /* found settings */
-        i=GWEN_DB_GetIntValue(dbT, "sortColumn", 0, -1);
-        if (i!=-1)
-          transactionListView->setSortColumn(i);
-        p=GWEN_DB_GetCharValue(dbT, "sortOrder", 0, "ascending");
-        if (p) {
-          if (strcasecmp(p, "ascending")==0)
-            transactionListView->setSortOrder(Qt::Ascending);
-          else
-            if (strcasecmp(p, "descending")==0)
-              transactionListView->setSortOrder(Qt::Descending);
-        }
-
-        for (i=0; i<transactionListView->columns(); i++) {
-          transactionListView->setColumnWidthMode(i, QListView::Manual);
-          j=GWEN_DB_GetIntValue(dbT, "columns", i, -1);
-          if (j!=-1)
-          transactionListView->setColumnWidth(i, j);
-        } /* for */
-      } // if transaction list data
-
-      dbT=GWEN_DB_GetGroup(db, GWEN_PATH_FLAGS_NAMEMUSTEXIST,
-                           "payeeList");
-      if (dbT) {
-        const char *p;
-
-        /* found settings */
-        i=GWEN_DB_GetIntValue(dbT, "sortColumn", 0, -1);
-        if (i!=-1)
-          payeeListView->setSortColumn(i);
-        p=GWEN_DB_GetCharValue(dbT, "sortOrder", 0, "ascending");
-        if (p) {
-          if (strcasecmp(p, "ascending")==0)
-            payeeListView->setSortOrder(Qt::Ascending);
-          else
-            if (strcasecmp(p, "descending")==0)
-              payeeListView->setSortOrder(Qt::Descending);
-        }
-
-        for (i=0; i<payeeListView->columns(); i++) {
-          payeeListView->setColumnWidthMode(i, QListView::Manual);
-          j=GWEN_DB_GetIntValue(dbT, "columns", i, -1);
-          if (j!=-1)
-          payeeListView->setColumnWidth(i, j);
-        } /* for */
-      } // if payee list data
-
-      dbT=GWEN_DB_GetGroup(db, GWEN_PATH_FLAGS_NAMEMUSTEXIST,
-                           "splitter");
-      if (dbT) {
-        int i, j;
-        QValueList<int> sizes;
-
-        /* found settings */
-        for (i=0; ; i++) {
-          j=GWEN_DB_GetIntValue(dbT, "sizes", i, -1);
-          if (j!=-1)
-            sizes.push_back(j);
-          else
-            break;
-        } /* for */
-
-        if (!sizes.empty())
-          splitter->setSizes(sizes);
-      } // if splitter settings
-    } /* if settings */
+  rv=_app->loadAppSubConfig("gui/dlgs/transactionFinder",
+			    &dbConfig,
+			    0);
+  if (rv<0) {
+    DBG_INFO(0, "here (%d)", rv);
   }
+  else {
+    GWEN_DB_NODE *dbT;
+    int w, h;
+
+    categoryListView->beginModify();
+
+    w=GWEN_DB_GetIntValue(dbConfig, "width", 0, 640);
+    h=GWEN_DB_GetIntValue(dbConfig, "height", 0, 480);
+    resize(w, h);
+
+    dbT=GWEN_DB_GetGroup(dbConfig, GWEN_PATH_FLAGS_NAMEMUSTEXIST, "accountList");
+    if (dbT) {
+      const char *p;
+
+      /* found settings */
+      i=GWEN_DB_GetIntValue(dbT, "sortColumn", 0, -1);
+      if (i!=-1)
+	accountListView->setSortColumn(i);
+      p=GWEN_DB_GetCharValue(dbT, "sortOrder", 0, "ascending");
+      if (p) {
+	if (strcasecmp(p, "ascending")==0)
+	  accountListView->setSortOrder(Qt::Ascending);
+	else
+	  if (strcasecmp(p, "descending")==0)
+	    accountListView->setSortOrder(Qt::Descending);
+      }
+      for (i=0; i<accountListView->columns(); i++) {
+	accountListView->setColumnWidthMode(i, QListView::Manual);
+	j=GWEN_DB_GetIntValue(dbT, "columns", i, -1);
+	if (j!=-1)
+	  accountListView->setColumnWidth(i, j);
+      } /* for */
+    } // if account list data
+
+    dbT=GWEN_DB_GetGroup(dbConfig, GWEN_PATH_FLAGS_NAMEMUSTEXIST, "categoryList");
+    if (dbT) {
+      const char *p;
+
+      /* found settings */
+      i=GWEN_DB_GetIntValue(dbT, "sortColumn", 0, -1);
+      if (i!=-1)
+	categoryListView->setSortColumn(i);
+      p=GWEN_DB_GetCharValue(dbT, "sortOrder", 0, "ascending");
+      if (p) {
+	if (strcasecmp(p, "ascending")==0)
+	  categoryListView->setSortOrder(Qt::Ascending);
+	else
+	  if (strcasecmp(p, "descending")==0)
+	    categoryListView->setSortOrder(Qt::Descending);
+      }
+      for (i=0; i<categoryListView->columns(); i++) {
+	categoryListView->setColumnWidthMode(i, QListView::Manual);
+	j=GWEN_DB_GetIntValue(dbT, "columns", i, -1);
+	if (j!=-1)
+	  categoryListView->setColumnWidth(i, j);
+      } /* for */
+    } // if category list data
+
+    dbT=GWEN_DB_GetGroup(dbConfig, GWEN_PATH_FLAGS_NAMEMUSTEXIST,
+			 "transactionList");
+    if (dbT) {
+      const char *p;
+
+      /* found settings */
+      i=GWEN_DB_GetIntValue(dbT, "sortColumn", 0, -1);
+      if (i!=-1)
+	transactionListView->setSortColumn(i);
+      p=GWEN_DB_GetCharValue(dbT, "sortOrder", 0, "ascending");
+      if (p) {
+	if (strcasecmp(p, "ascending")==0)
+	  transactionListView->setSortOrder(Qt::Ascending);
+	else
+	  if (strcasecmp(p, "descending")==0)
+	    transactionListView->setSortOrder(Qt::Descending);
+      }
+
+      for (i=0; i<transactionListView->columns(); i++) {
+	transactionListView->setColumnWidthMode(i, QListView::Manual);
+	j=GWEN_DB_GetIntValue(dbT, "columns", i, -1);
+	if (j!=-1)
+	  transactionListView->setColumnWidth(i, j);
+      } /* for */
+    } // if transaction list data
+
+    dbT=GWEN_DB_GetGroup(dbConfig, GWEN_PATH_FLAGS_NAMEMUSTEXIST,
+			 "payeeList");
+    if (dbT) {
+      const char *p;
+
+      /* found settings */
+      i=GWEN_DB_GetIntValue(dbT, "sortColumn", 0, -1);
+      if (i!=-1)
+	payeeListView->setSortColumn(i);
+      p=GWEN_DB_GetCharValue(dbT, "sortOrder", 0, "ascending");
+      if (p) {
+	if (strcasecmp(p, "ascending")==0)
+	  payeeListView->setSortOrder(Qt::Ascending);
+	else
+	  if (strcasecmp(p, "descending")==0)
+	    payeeListView->setSortOrder(Qt::Descending);
+      }
+
+      for (i=0; i<payeeListView->columns(); i++) {
+	payeeListView->setColumnWidthMode(i, QListView::Manual);
+	j=GWEN_DB_GetIntValue(dbT, "columns", i, -1);
+	if (j!=-1)
+	  payeeListView->setColumnWidth(i, j);
+      } /* for */
+    } // if payee list data
+
+    dbT=GWEN_DB_GetGroup(dbConfig, GWEN_PATH_FLAGS_NAMEMUSTEXIST,
+			 "splitter");
+    if (dbT) {
+      int i, j;
+      QValueList<int> sizes;
+
+      /* found settings */
+      for (i=0; ; i++) {
+	j=GWEN_DB_GetIntValue(dbT, "sizes", i, -1);
+	if (j!=-1)
+	  sizes.push_back(j);
+	else
+	  break;
+      } /* for */
+
+      if (!sizes.empty())
+	splitter->setSizes(sizes);
+    } // if splitter settings
+    GWEN_DB_Group_free(dbConfig);
+  } /* if settings */
 
   /* fill with data from application */
   accountListView->addAccounts(_app->getAppAccounts());
@@ -351,23 +355,22 @@ bool TransactionFinder::init(){
 
 
 bool TransactionFinder::fini(){
-  GWEN_DB_NODE *db;
+  GWEN_DB_NODE *dbConfig;
   GWEN_DB_NODE *dbT;
   int i, j;
   QValueList<int> sizes;
+  int rv;
 
-  db=_app->getAppData();
-  assert(db);
-  db=GWEN_DB_GetGroup(db, GWEN_DB_FLAGS_OVERWRITE_GROUPS,
-                      "gui/dlgs/transactionFinder");
-  assert(db);
-  GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_DEFAULT,
+  dbConfig=GWEN_DB_Group_new("config");
+  assert(dbConfig);
+
+  GWEN_DB_SetIntValue(dbConfig, GWEN_DB_FLAGS_DEFAULT,
                       "width", width());
-  GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_DEFAULT,
+  GWEN_DB_SetIntValue(dbConfig, GWEN_DB_FLAGS_DEFAULT,
                       "height", height());
 
   // store account list info
-  dbT=GWEN_DB_GetGroup(db, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "accountList");
+  dbT=GWEN_DB_GetGroup(dbConfig, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "accountList");
   GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_DEFAULT,
                       "sortColumn", accountListView->sortColumn());
   switch(accountListView->sortOrder()) {
@@ -390,7 +393,7 @@ bool TransactionFinder::fini(){
   } /* for */
 
   // store payee list info
-  dbT=GWEN_DB_GetGroup(db, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "payeeList");
+  dbT=GWEN_DB_GetGroup(dbConfig, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "payeeList");
   GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_DEFAULT,
                       "sortColumn", payeeListView->sortColumn());
   switch(payeeListView->sortOrder()) {
@@ -413,7 +416,7 @@ bool TransactionFinder::fini(){
   } /* for */
 
   // store category list info
-  dbT=GWEN_DB_GetGroup(db, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "categoryList");
+  dbT=GWEN_DB_GetGroup(dbConfig, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "categoryList");
   GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_DEFAULT,
                       "sortColumn", categoryListView->sortColumn());
   switch(categoryListView->sortOrder()) {
@@ -436,7 +439,7 @@ bool TransactionFinder::fini(){
   } /* for */
 
   // store transaction list info
-  dbT=GWEN_DB_GetGroup(db, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "transactionList");
+  dbT=GWEN_DB_GetGroup(dbConfig, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "transactionList");
   GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_DEFAULT,
                       "sortColumn", transactionListView->sortColumn());
   switch(transactionListView->sortOrder()) {
@@ -459,7 +462,7 @@ bool TransactionFinder::fini(){
   } /* for */
 
   /* save splitter settings */
-  dbT=GWEN_DB_GetGroup(db,
+  dbT=GWEN_DB_GetGroup(dbConfig,
                        GWEN_DB_FLAGS_OVERWRITE_GROUPS,
                        "splitter");
   sizes=splitter->sizes();
@@ -470,7 +473,14 @@ bool TransactionFinder::fini(){
     ++it;
   }
 
-
+  rv=_app->saveAppSubConfig("gui/dlgs/transactionFinder",
+			    dbConfig,
+			    0);
+  GWEN_DB_Group_free(dbConfig);
+  if (rv<0) {
+    DBG_INFO(0, "here (%d)", rv);
+    return false;
+  }
   return true;
 }
 
