@@ -1,7 +1,7 @@
 /***************************************************************************
  $RCSfile$
                              -------------------
-    cvs         : $Id: kbanking.cpp 435 2008-02-11 13:42:42Z martin $
+    cvs         : $Id$
     begin       : Mon Mar 01 2004
     copyright   : (C) 2004 by Martin Preuss
     email       : martin@libchipcard.de
@@ -284,8 +284,7 @@ void KBanking::rebuildExportMenu(){
 
 
 int KBanking::init(){
-  GWEN_DB_NODE *db;
-  GWEN_DB_NODE *dbCerts;
+  GWEN_DB_NODE *dbConfig;
   int rv;
   uint32_t lastVersion;
 
@@ -320,21 +319,39 @@ int KBanking::init(){
     w.gatherInfoFromPurpose();
   }
 
-  db=getAppData();
-  assert(db);
-  db=GWEN_DB_GetGroup(db, GWEN_DB_FLAGS_DEFAULT,
-                      "kbanking");
-  assert(db);
-  _lastReportId=GWEN_DB_GetIntValue(db, "lastReportId", 0, 0);
-  _useColoursForCategoryText=GWEN_DB_GetIntValue(db,
-                                                 "useColoursForCategoryText",
-                                                 0, 1)!=0;
-
+  rv=loadAppSubConfig("kbanking/app",
+		      &dbConfig,
+		      0);
+  if (rv<0) {
+    DBG_INFO(0, "here (%d)", rv);
+  }
+  else {
+    _lastReportId=GWEN_DB_GetIntValue(dbConfig, "lastReportId", 0, 0);
+    _useColoursForCategoryText=GWEN_DB_GetIntValue(dbConfig,
+						   "useColoursForCategoryText",
+						   0, 1)!=0;
+  }
   _sampleReportModules();
 
-  dbCerts=GWEN_DB_GetGroup(db, GWEN_PATH_FLAGS_NAMEMUSTEXIST, "certs");
-  if (dbCerts)
-    getGui()->setDbCerts(GWEN_DB_Group_dup(dbCerts));
+  rv=lockSharedConfig("certs", 0);
+  if (rv<0) {
+    DBG_ERROR(0, "ERROR: Could not lock certs db (%d)", rv);
+  }
+  else {
+    GWEN_DB_NODE *dbCerts=NULL;
+
+    rv=loadSharedConfig("certs", &dbCerts, 0);
+    if (rv<0) {
+      DBG_ERROR(0, "ERROR: Could not load certs (%d)", rv);
+    }
+    else {
+      getGui()->setDbCerts(dbCerts);
+    }
+    rv=unlockSharedConfig("certs", 0);
+    if (rv<0) {
+      DBG_ERROR(0, "ERROR: Could not unlock certs db (%d)", rv);
+    }
+  }
 
   return 0;
 }
@@ -342,29 +359,48 @@ int KBanking::init(){
 
 
 int KBanking::fini(){
-  GWEN_DB_NODE *db;
-  GWEN_DB_NODE *db2;
+  GWEN_DB_NODE *dbConfig;
   GWEN_DB_NODE *dbCerts;
+  int rv;
 
-  db=getAppData();
-  assert(db);
-  db=GWEN_DB_GetGroup(db, GWEN_DB_FLAGS_DEFAULT,
-                      "kbanking");
-  assert(db);
-  GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS,
+  dbConfig=GWEN_DB_Group_new("config");
+  assert(dbConfig);
+
+  GWEN_DB_SetIntValue(dbConfig, GWEN_DB_FLAGS_OVERWRITE_VARS,
                       "lastReportId",
                       _lastReportId);
 
-  GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS,
-                      "useColoursForCategoryText",
-                      _useColoursForCategoryText?1:0);
+  GWEN_DB_SetIntValue(dbConfig, GWEN_DB_FLAGS_OVERWRITE_VARS,
+		      "useColoursForCategoryText",
+		      _useColoursForCategoryText?1:0);
+
+  rv=saveAppSubConfig("kbanking/app",
+		      dbConfig,
+		      0);
+  GWEN_DB_Group_free(dbConfig);
+  if (rv<0) {
+    DBG_INFO(0, "here (%d)", rv);
+    return rv;
+  }
 
   /* store certificates */
-  dbCerts=GWEN_DB_GetGroup(db, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "certs");
-  assert(dbCerts);
-
-  db2=getGui()->getDbCerts();
-  GWEN_DB_AddGroupChildren(dbCerts, db2);
+  dbCerts=getGui()->getDbCerts();
+  if (dbCerts) {
+    rv=lockSharedConfig("certs", 0);
+    if (rv<0) {
+      DBG_ERROR(0, "ERROR: Could not lock certs db (%d)", rv);
+    }
+    else {
+      rv=saveSharedConfig("certs", dbCerts, 0);
+      if (rv<0) {
+	DBG_ERROR(0, "ERROR: Could not save certs (%d)", rv);
+      }
+      rv=unlockSharedConfig("certs", 0);
+      if (rv<0) {
+	fprintf(stderr, "ERROR: Could not unlock certs db (%d)\n", rv);
+      }
+    }
+  }
 
   return App::fini();
 }
@@ -418,40 +454,47 @@ std::list<Report*> KBanking::getReports(){
 
 
 GWEN_DB_NODE *KBanking::getReportProfiles(Report *r) {
-  GWEN_DB_NODE *db;
+  GWEN_DB_NODE *dbConfig=NULL;
   GWEN_BUFFER *nbuf;
   std::string name;
+  std::string path;
+  int rv;
 
   name=QStringToUtf8String(r->name());
-  db=getAppData();
-  assert(db);
-  db=GWEN_DB_GetGroup(db, GWEN_DB_FLAGS_DEFAULT,
-                      "kbanking/reports");
-  assert(db);
+  path="kbanking/reports";
+
   nbuf=GWEN_Buffer_new(0, 256, 0, 1);
   if (GWEN_Text_EscapeToBuffer(name.c_str(), nbuf)) {
     DBG_ERROR(0, "Could not escape report name");
     GWEN_Buffer_free(nbuf);
     return 0;
   }
-  GWEN_Buffer_AppendString(nbuf, "/profiles");
-  db=GWEN_DB_GetGroup(db, GWEN_DB_FLAGS_DEFAULT,
-                      GWEN_Buffer_GetStart(nbuf));
-  assert(db);
+  path+="/";
+  path+=GWEN_Buffer_GetStart(nbuf);
+  path+="/profiles";
   GWEN_Buffer_free(nbuf);
 
-  return db;
+  rv=loadAppSubConfig(path.c_str(),
+		      &dbConfig,
+		      0);
+  if (rv<0) {
+    DBG_INFO(0, "here (%d)", rv);
+    return NULL;
+  }
+
+  return dbConfig;
 }
 
 
 
 GWEN_DB_NODE *KBanking::findReportProfile(Report *r, const char *id){
+  GWEN_DB_NODE *dbConfig;
   GWEN_DB_NODE *db;
 
-  db=getReportProfiles(r);
-  if (!db)
+  dbConfig=getReportProfiles(r);
+  if (!dbConfig)
     return 0;
-  db=GWEN_DB_FindFirstGroup(db, "profile");
+  db=GWEN_DB_FindFirstGroup(dbConfig, "profile");
   while(db) {
     const char *s;
 
@@ -460,31 +503,68 @@ GWEN_DB_NODE *KBanking::findReportProfile(Report *r, const char *id){
       DBG_WARN(0, "Report with no id detected");
     }
     else {
-      if (strcasecmp(s, id)==0)
-        return db;
+      if (strcasecmp(s, id)==0) {
+	GWEN_DB_UnlinkGroup(db);
+	GWEN_DB_Group_free(dbConfig);
+	return db;
+      }
     }
     db=GWEN_DB_FindNextGroup(db, "profile");
   } // while
+  GWEN_DB_Group_free(dbConfig);
 
-  return 0;
+  return NULL;
 }
 
 
 
 int KBanking::addReportProfile(Report *r, GWEN_DB_NODE *dbProfile){
-  GWEN_DB_NODE *db;
-  char numbuf[32];
+  GWEN_DB_NODE *dbConfig=NULL;
+  GWEN_BUFFER *nbuf;
+  std::string name;
+  std::string path;
+  int rv;
 
-  db=getReportProfiles(r);
-  if (!db)
-    return -1;
+  name=QStringToUtf8String(r->name());
+  path="kbanking/reports";
+  nbuf=GWEN_Buffer_new(0, 256, 0, 1);
+  if (GWEN_Text_EscapeToBuffer(name.c_str(), nbuf)) {
+    DBG_ERROR(0, "Could not escape report name");
+    GWEN_Buffer_free(nbuf);
+    return GWEN_ERROR_INVALID;
+  }
+  path+="/";
+  path+=GWEN_Buffer_GetStart(nbuf);
+  path+="/profiles";
+  GWEN_Buffer_free(nbuf);
 
-  snprintf(numbuf, sizeof(numbuf), "%d", ++_lastReportId);
-  GWEN_DB_SetCharValue(dbProfile, GWEN_DB_FLAGS_OVERWRITE_VARS,
-                       "id", numbuf);
-  if (strcasecmp(GWEN_DB_GroupName(dbProfile), "profile")!=0)
-    GWEN_DB_GroupRename(dbProfile, "profile");
-  GWEN_DB_AddGroup(db, dbProfile);
+  rv=loadAppSubConfig(path.c_str(),
+		      &dbConfig,
+		      0);
+  if (rv<0) {
+    DBG_INFO(0, "here (%d)", rv);
+    return rv;
+  }
+  else {
+    char numbuf[32];
+    GWEN_DB_NODE *dbNew=GWEN_DB_GetGroup(dbConfig,
+					 GWEN_PATH_FLAGS_CREATE_GROUP,
+					 "profile");
+    assert(dbNew);
+    GWEN_DB_AddGroupChildren(dbNew, dbProfile);
+    snprintf(numbuf, sizeof(numbuf), "%d", ++_lastReportId);
+    GWEN_DB_SetCharValue(dbNew, GWEN_DB_FLAGS_OVERWRITE_VARS,
+			 "id", numbuf);
+
+    rv=saveAppSubConfig(path.c_str(),
+			dbConfig,
+			0);
+    GWEN_DB_Group_free(dbConfig);
+    if (rv<0) {
+      DBG_INFO(0, "here (%d)", rv);
+      return rv;
+    }
+  }
 
   return 0;
 }
@@ -493,8 +573,7 @@ int KBanking::addReportProfile(Report *r, GWEN_DB_NODE *dbProfile){
 
 void KBanking::removeReportProfile(Report *r,
                                    GWEN_DB_NODE *dbProfile){
-  GWEN_DB_UnlinkGroup(dbProfile);
-  GWEN_DB_Group_free(dbProfile);
+  DBG_ERROR(0, "Not yet implemented");
 }
 
 
@@ -1643,23 +1722,4 @@ int KBanking::dequeueJob(AB_JOB *j) {
 
 
 
-QString KBanking::sanitizedNumber(const QString &qs) {
-  if (!qs.isEmpty()) {
-    QString qs2;
-    unsigned int i;
-
-    for (i=0; i<qs.length(); i++) {
-      if (qs[i].isDigit())
-	qs2+=qs[i];
-    }
-    return qs2;
-  }
-  else
-    return qs;
-}
-
-
-
-
-
 
