@@ -1,7 +1,7 @@
 /***************************************************************************
  $RCSfile: app.cpp,v $
                              -------------------
-    cvs         : $Id: app.cpp 435 2008-02-11 13:42:42Z martin $
+    cvs         : $Id$
     begin       : Mon Mar 01 2004
     copyright   : (C) 2004 by Martin Preuss
     email       : martin@libchipcard.de
@@ -82,7 +82,7 @@ std::list<Account*> &App::getAppAccounts(){
 
 
 int App::init() {
-  GWEN_DB_NODE *db;
+  GWEN_DB_NODE *dbConfig=NULL;
   GWEN_DB_NODE *dbT;
   int rv;
   uint32_t currentVersion;
@@ -95,15 +95,18 @@ int App::init() {
     return rv;
   }
 
-  rv=AB_Banking_OnlineInit(getCInterface());
+  rv=AB_Banking_OnlineInit(getCInterface(), 0);
   if (rv) {
     DBG_ERROR(0, "here (%d)", rv);
     return rv;
   }
 
   /* read configuration */
-  db=getAppData();
-  db=GWEN_DB_GetGroup(db, GWEN_DB_FLAGS_DEFAULT, "prg");
+  rv=loadAppSubConfig("prg", &dbConfig, 0);
+  if (rv<0) {
+    DBG_ERROR(0, "here (%d)", rv);
+    return rv;
+  }
 
   currentVersion=
     (AQBANKING_KDE_VERSION_MAJOR<<24)+
@@ -111,41 +114,41 @@ int App::init() {
     (AQBANKING_KDE_VERSION_PATCHLEVEL<<8)+
     AQBANKING_KDE_VERSION_BUILD;
 
-  _lastVersion=GWEN_DB_GetIntValue(db, "lastVersion", 0,
+  _lastVersion=GWEN_DB_GetIntValue(dbConfig, "lastVersion", 0,
                                    APP_LAST_VERSION_NONE);
 
-  _fuzzyThreshold=GWEN_DB_GetIntValue(db, "fuzzyThreshold", 0,
+  _fuzzyThreshold=GWEN_DB_GetIntValue(dbConfig, "fuzzyThreshold", 0,
                                       APP_FUZZY_THRESHOLD);
 
-  _sensitiveFuzzyThreshold=GWEN_DB_GetIntValue(db,
+  _sensitiveFuzzyThreshold=GWEN_DB_GetIntValue(dbConfig,
                                                "sensitiveFuzzyThreshold", 0,
                                                APP_FUZZY_SENSITIVE_THRESHOLD);
 
-  dbT=GWEN_DB_GetGroup(db, GWEN_DB_FLAGS_DEFAULT,
+  dbT=GWEN_DB_GetGroup(dbConfig, GWEN_DB_FLAGS_DEFAULT,
                        "tMatcherRules");
   if (dbT)
     _xaMatcherRules=GWEN_DB_Group_dup(dbT);
   else
     _xaMatcherRules=GWEN_DB_Group_new("tMatcherRules");
 
-  dbT=GWEN_DB_GetGroup(db, GWEN_DB_FLAGS_DEFAULT,
+  dbT=GWEN_DB_GetGroup(dbConfig, GWEN_DB_FLAGS_DEFAULT,
                        "exportRules");
   if (dbT)
     _exportRules=GWEN_DB_Group_dup(dbT);
   else
     _exportRules=GWEN_DB_Group_new("exportRules");
 
-  _lastUniqueId=GWEN_DB_GetIntValue(db, "lastUniqueId", 0, 0);
-  _lastPayeeId=GWEN_DB_GetIntValue(db, "lastPayeeId", 0, 0);
-  _lastCategoryId=GWEN_DB_GetIntValue(db, "lastCategoryId", 0, 0);
+  _lastUniqueId=GWEN_DB_GetIntValue(dbConfig, "lastUniqueId", 0, 0);
+  _lastPayeeId=GWEN_DB_GetIntValue(dbConfig, "lastPayeeId", 0, 0);
+  _lastCategoryId=GWEN_DB_GetIntValue(dbConfig, "lastCategoryId", 0, 0);
 
-  _optionAutoAssignPayee=GWEN_DB_GetIntValue(db, "autoAssignPayee", 0, 1);
-  _optionAutoAskForPayee=GWEN_DB_GetIntValue(db, "autoAskForPayee", 0, 0);
+  _optionAutoAssignPayee=GWEN_DB_GetIntValue(dbConfig, "autoAssignPayee", 0, 1);
+  _optionAutoAskForPayee=GWEN_DB_GetIntValue(dbConfig, "autoAskForPayee", 0, 0);
 
-  _optionAutoAssignCategory=GWEN_DB_GetIntValue(db, "autoAssignCategory", 0, 1);
-  _optionAutoAskForCategory=GWEN_DB_GetIntValue(db, "autoAskForCategory", 0, 0);
+  _optionAutoAssignCategory=GWEN_DB_GetIntValue(dbConfig, "autoAssignCategory", 0, 1);
+  _optionAutoAskForCategory=GWEN_DB_GetIntValue(dbConfig, "autoAskForCategory", 0, 0);
 
-  dbT=GWEN_DB_GetGroup(db, GWEN_PATH_FLAGS_NAMEMUSTEXIST,
+  dbT=GWEN_DB_GetGroup(dbConfig, GWEN_PATH_FLAGS_NAMEMUSTEXIST,
                        "accounts");
   if (dbT) {
     GWEN_DB_NODE *dbAccount;
@@ -199,14 +202,15 @@ int App::init() {
   rv=loadPayees();
   if (rv) {
     DBG_ERROR(0, "Error loading payees (%d)", rv);
+    GWEN_DB_Group_free(dbConfig);
     return rv;
   }
 
   if (_payees.empty()) {
     /* storing payees within the AqBanking configuration is now obsolete,
      * so we only try to read from it if there were no payees in the new
-     * payee.db file */
-    dbT=GWEN_DB_GetGroup(db, GWEN_PATH_FLAGS_NAMEMUSTEXIST,
+     * payee.dbConfig file */
+    dbT=GWEN_DB_GetGroup(dbConfig, GWEN_PATH_FLAGS_NAMEMUSTEXIST,
 			 "payees");
     if (dbT) {
       GWEN_DB_NODE *dbPayee;
@@ -232,11 +236,13 @@ int App::init() {
   rv=loadCategories();
   if (rv) {
     DBG_INFO(0, "Error loading categories (%d)", rv);
+    GWEN_DB_Group_free(dbConfig);
     return rv;
   }
   if (_categories.empty()) {
-    rv=_loadCategoryFile(CATEGORYDIR "/gnucash-full.db");
+    rv=_loadCategoryFile(CATEGORYDIR "/gnucash-full.dbConfig");
     DBG_INFO(0, "Error loading categories (%d)", rv);
+    GWEN_DB_Group_free(dbConfig);
     return rv;
   }
   if (_categories.empty()) {
@@ -285,7 +291,7 @@ int App::init() {
   }
 
   /* read transfers (only old transfers, new ones are stored with accounts) */
-  dbT=GWEN_DB_GetGroup(db, GWEN_PATH_FLAGS_NAMEMUSTEXIST,
+  dbT=GWEN_DB_GetGroup(dbConfig, GWEN_PATH_FLAGS_NAMEMUSTEXIST,
                        "transfers");
   if (dbT) {
     GWEN_DB_NODE *dbTransfer;
@@ -305,8 +311,11 @@ int App::init() {
   /* sync account list */
   if (!updateAccountList()) {
     DBG_ERROR(0, "Could not update account list (%d)", rv);
+    GWEN_DB_Group_free(dbConfig);
     return GWEN_ERROR_GENERIC;
   }
+
+  GWEN_DB_Group_free(dbConfig);
   return 0;
 }
 
@@ -318,63 +327,61 @@ int App::fini() {
   std::list<RefPointer<StandingOrder> >::iterator its;
   std::list<Payee*>::iterator pit;
   std::list<Category*>::iterator cit;
-  GWEN_DB_NODE *db;
+  GWEN_DB_NODE *dbConfig;
   GWEN_DB_NODE *dbT;
   int rv;
 
-  db=getAppData();
-  db=GWEN_DB_GetGroup(db, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "prg");
-  assert(db);
+  dbConfig=GWEN_DB_Group_new("qbankmanager");
 
-  GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS,
+  GWEN_DB_SetIntValue(dbConfig, GWEN_DB_FLAGS_OVERWRITE_VARS,
                       "lastVersion",
                       (AQBANKING_KDE_VERSION_MAJOR<<24)+
                       (AQBANKING_KDE_VERSION_MINOR<<16)+
                       (AQBANKING_KDE_VERSION_PATCHLEVEL<<8)+
                       AQBANKING_KDE_VERSION_BUILD);
 
-  GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS,
+  GWEN_DB_SetIntValue(dbConfig, GWEN_DB_FLAGS_OVERWRITE_VARS,
                       "lastPayeeId", _lastPayeeId);
-  GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS,
+  GWEN_DB_SetIntValue(dbConfig, GWEN_DB_FLAGS_OVERWRITE_VARS,
                       "lastCategoryId", _lastCategoryId);
-  GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS,
+  GWEN_DB_SetIntValue(dbConfig, GWEN_DB_FLAGS_OVERWRITE_VARS,
 		      "lastUniqueId", _lastUniqueId);
 
-  GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS,
+  GWEN_DB_SetIntValue(dbConfig, GWEN_DB_FLAGS_OVERWRITE_VARS,
                       "fuzzyThreshold",
                       _fuzzyThreshold);
-  GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS,
+  GWEN_DB_SetIntValue(dbConfig, GWEN_DB_FLAGS_OVERWRITE_VARS,
                       "sensitiveFuzzyThreshold",
                       _sensitiveFuzzyThreshold);
 
-  GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS,
+  GWEN_DB_SetIntValue(dbConfig, GWEN_DB_FLAGS_OVERWRITE_VARS,
                       "autoAssignPayee",
                       _optionAutoAssignPayee);
-  GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS,
+  GWEN_DB_SetIntValue(dbConfig, GWEN_DB_FLAGS_OVERWRITE_VARS,
                       "autoAskForPayee",
                       _optionAutoAskForPayee);
 
-  GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS,
+  GWEN_DB_SetIntValue(dbConfig, GWEN_DB_FLAGS_OVERWRITE_VARS,
                       "autoAssignCategory",
                       _optionAutoAssignCategory);
-  GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS,
+  GWEN_DB_SetIntValue(dbConfig, GWEN_DB_FLAGS_OVERWRITE_VARS,
                       "autoAskForCategory",
                       _optionAutoAskForCategory);
 
-  dbT=GWEN_DB_GetGroup(db, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "tMatcherRules");
+  dbT=GWEN_DB_GetGroup(dbConfig, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "tMatcherRules");
   assert(dbT);
   GWEN_DB_AddGroupChildren(dbT, _xaMatcherRules);
   GWEN_DB_Group_free(_xaMatcherRules);
   _xaMatcherRules=0;
 
-  dbT=GWEN_DB_GetGroup(db, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "exportRules");
+  dbT=GWEN_DB_GetGroup(dbConfig, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "exportRules");
   assert(dbT);
   GWEN_DB_AddGroupChildren(dbT, _exportRules);
   GWEN_DB_Group_free(_exportRules);
   _exportRules=0;
 
   // save accounts
-  dbT=GWEN_DB_GetGroup(db, GWEN_DB_FLAGS_DEFAULT, "accounts");
+  dbT=GWEN_DB_GetGroup(dbConfig, GWEN_DB_FLAGS_DEFAULT, "accounts");
   for (it=_accounts.begin(); it!=_accounts.end(); it++) {
     GWEN_DB_NODE *dbAccount;
 
@@ -383,6 +390,7 @@ int App::fini() {
     assert(dbAccount);
     if (!(*it)->toDb(dbAccount)) {
       DBG_ERROR(0, "Could not save account");
+      GWEN_DB_Group_free(dbConfig);
       return GWEN_ERROR_GENERIC;
     }
   }
@@ -391,6 +399,7 @@ int App::fini() {
   rv=savePayees();
   if (rv) {
     DBG_INFO(0, "Error saving payees (%d)", rv);
+    GWEN_DB_Group_free(dbConfig);
     return rv;
   }
 
@@ -398,6 +407,7 @@ int App::fini() {
   rv=saveCategories();
   if (rv) {
     DBG_INFO(0, "Error saving categories (%d)", rv);
+    GWEN_DB_Group_free(dbConfig);
     return rv;
   }
 
@@ -414,7 +424,16 @@ int App::fini() {
     delete *cit;
   _categories.clear();
 
-  rv=AB_Banking_OnlineFini(getCInterface());
+  rv=saveAppSubConfig("prg", dbConfig, 0);
+  if (rv<0) {
+    DBG_INFO(0, "Error saving application data");
+    GWEN_DB_Group_free(dbConfig);
+    return rv;
+  }
+  GWEN_DB_Group_free(dbConfig);
+
+  /* finalize online banking stuff */
+  rv=AB_Banking_OnlineFini(getCInterface(), 0);
   if (rv)
     return rv;
 
@@ -1905,6 +1924,118 @@ int App::exportTransactions(GWEN_DB_NODE *dbRule,
     AB_ImExporterContext_AddTransaction(ctx, bt);
   } /* for */
 
+  AB_Banking_FillGapsInImExporterContext(getCInterface(), ctx);
+
+  /* let exporter work */
+  rv=AB_ImExporter_Export(exporter,
+			  ctx,
+			  io,
+			  dbProfile,
+			  0);
+  AB_ImExporterContext_free(ctx);
+  if (rv<0) {
+    DBG_ERROR(0, "Export plugin signalled an error (%d)", rv);
+    GWEN_Io_Layer_free(io);
+    close(fd);
+    return rv;
+  }
+  else {
+    /* flush */
+    rv=GWEN_Io_Layer_DisconnectRecursively(io, NULL, 0, 0, 2000);
+    if (rv<0) {
+      DBG_ERROR(0, "Error on disconnect (%d)", rv);
+      GWEN_Io_Layer_free(io);
+      close(fd);
+      return rv;
+    }
+    GWEN_Io_Layer_free(io);
+
+    if (close(fd)) {
+      DBG_ERROR(0, "close(): %s", strerror(errno));
+      return GWEN_ERROR_IO;
+    }
+  }
+  return 0;
+}
+
+
+
+int App::exportTransfers(GWEN_DB_NODE *dbRule,
+			 const std::list<RefPointer<Transfer> > &tl){
+  AB_IMEXPORTER_CONTEXT *ctx;
+  std::list<RefPointer<Transfer> >::const_iterator it;
+  AB_IMEXPORTER *exporter;
+  const char *exporterName;
+  GWEN_DB_NODE *dbProfile;
+  int rv;
+  GWEN_IO_LAYER *io;
+  const char *fname;
+  int fd;
+
+  /* get exporter name */
+  exporterName=GWEN_DB_GetCharValue(dbRule, "exporterText", 0, 0);
+  if (!exporterName) {
+    DBG_ERROR(0, "No exporter set");
+    return GWEN_ERROR_INVALID;
+  }
+
+  /* get profile */
+  dbProfile=GWEN_DB_GetGroup(dbRule, GWEN_PATH_FLAGS_NAMEMUSTEXIST,
+                             "profile");
+  if (!dbProfile) {
+    DBG_ERROR(0, "No export profile set");
+    return GWEN_ERROR_INVALID;
+  }
+
+  /* get exporter */
+  exporter=AB_Banking_GetImExporter(getCInterface(), exporterName);
+  if (!exporter) {
+    DBG_ERROR(0, "Exporter %s not found", exporterName);
+    return GWEN_ERROR_INVALID;
+  }
+
+  /* check for file */
+  fname=GWEN_DB_GetCharValue(dbRule, "outfileText", 0, 0);
+  if (!fname) {
+    DBG_ERROR(0, "No file name set");
+    return GWEN_ERROR_INVALID;
+  }
+
+  /* open file */
+  fd=open(fname,
+          O_RDWR | O_TRUNC | O_CREAT,
+#ifdef WIN32
+	  0
+#else
+	  S_IRUSR | S_IWUSR
+#endif
+	 );
+  if (fd==-1) {
+    DBG_ERROR(0, "open(%s): %s", fname, strerror(errno));
+    return GWEN_ERROR_GENERIC;
+  }
+
+  io=GWEN_Io_LayerFile_new(0, fd);
+  GWEN_Io_Layer_AddFlags(io, GWEN_IO_LAYER_FLAGS_DONTCLOSE);
+  rv=GWEN_Io_Manager_RegisterLayer(io);
+  if (rv<0) {
+    DBG_ERROR(0, "Error registering layer (%d)", rv);
+    close(fd);
+    return rv;
+  }
+
+  /* add transactions to context */
+  ctx=AB_ImExporterContext_new();
+  for (it=tl.begin(); it!=tl.end(); it++) {
+    AB_TRANSACTION *bt;
+
+    bt=(*it).ref().toBankingTransaction();
+    assert(bt);
+    AB_ImExporterContext_AddTransaction(ctx, bt);
+  } /* for */
+
+  AB_Banking_FillGapsInImExporterContext(getCInterface(), ctx);
+
   /* let exporter work */
   rv=AB_ImExporter_Export(exporter,
 			  ctx,
