@@ -1,7 +1,7 @@
 /***************************************************************************
  $RCSfile: timport.cpp,v $
                              -------------------
-    cvs         : $Id: timport.cpp 433 2008-01-12 08:58:57Z martin $
+    cvs         : $Id$
     begin       : Mon Mar 01 2004
     copyright   : (C) 2004 by Martin Preuss
     email       : martin@libchipcard.de
@@ -174,7 +174,8 @@ TransactionImporter::~TransactionImporter(){
 Account*
 TransactionImporter::askUserForAccount(const std::string &bankCode,
 				       const std::string &accountNumber,
-				       const std::string &accountName){
+				       const std::string &accountName,
+				       AB_ACCOUNT_TYPE t){
   return 0;
 }
 
@@ -248,7 +249,8 @@ bool TransactionImporter::sortNewTransactions() {
     currentAccount=
       getAccountData(AB_ImExporterAccountInfo_GetBankCode(ai),
 		     AB_ImExporterAccountInfo_GetAccountNumber(ai),
-		     AB_ImExporterAccountInfo_GetAccountName(ai));
+		     AB_ImExporterAccountInfo_GetAccountName(ai),
+		     AB_ImExporterAccountInfo_GetType(ai));
     if (!currentAccount) {
       // TODO: ask user whether he wants to abort
       return false;
@@ -904,7 +906,8 @@ bool TransactionImporter::importAccounts(){
 
     ad=getAccountData(AB_ImExporterAccountInfo_GetBankCode(ai),
 		      AB_ImExporterAccountInfo_GetAccountNumber(ai),
-		      AB_ImExporterAccountInfo_GetAccountName(ai));
+		      AB_ImExporterAccountInfo_GetAccountName(ai),
+		      AB_ImExporterAccountInfo_GetType(ai));
     if (!ad) {
       /* account not found, TODO: ask the user to add it */
       DBG_ERROR(0, "Account \"%s/%s\" not found",
@@ -1085,7 +1088,8 @@ bool TransactionImporter::importContext(){
 TransactionImporter::AccountData*
 TransactionImporter::getAccountData(const char *s1,
 				    const char *s2,
-				    const char *s3) {
+				    const char *s3,
+				    AB_ACCOUNT_TYPE t) {
   std::list<AccountData*>::iterator it;
   std::string bankCode;
   std::string accountNumber;
@@ -1141,7 +1145,7 @@ TransactionImporter::getAccountData(const char *s1,
   if (!a) {
     // TODO: check for aliases
     // TODO: ask the user for alias
-    a=askUserForAccount(bankCode, accountNumber, accountName);
+    a=askUserForAccount(bankCode, accountNumber, accountName, t);
     if (!a) {
       DBG_NOTICE(0, "Account not found");
       return NULL;
