@@ -1,7 +1,7 @@
 /***************************************************************************
  $RCSfile$
                              -------------------
-    cvs         : $Id: transaction.cpp 417 2007-10-12 15:00:16Z martin $
+    cvs         : $Id$
     begin       : Mon Mar 01 2004
     copyright   : (C) 2004 by Martin Preuss
     email       : martin@libchipcard.de
@@ -81,6 +81,13 @@ Transaction::Transaction(const AB_TRANSACTION *t)
   p=AB_Transaction_GetLocalBankCode(t);
   if (p)
     _localBankCode=p;
+  p=AB_Transaction_GetLocalIban(t);
+  if (p)
+    _localIban=p;
+  p=AB_Transaction_GetLocalBic(t);
+  if (p)
+    _localBic=p;
+
   p=AB_Transaction_GetLocalAccountNumber(t);
   if (p)
     _localAccountNumber=p;
@@ -112,6 +119,9 @@ Transaction::Transaction(const AB_TRANSACTION *t)
   p=AB_Transaction_GetRemoteIban(t);
   if (p)
     _remoteIban=p;
+  p=AB_Transaction_GetRemoteBic(t);
+  if (p)
+    _remoteBic=p;
 
   sl=AB_Transaction_GetRemoteName(t);
   if (sl) {
@@ -198,6 +208,10 @@ AB_TRANSACTION *Transaction::toBankingTransaction() {
     AB_Transaction_SetLocalBankCode(t, _localBankCode.c_str());
   if (!_localAccountNumber.empty())
     AB_Transaction_SetLocalAccountNumber(t, _localAccountNumber.c_str());
+  if (!_localIban.empty())
+    AB_Transaction_SetRemoteIban(t, _localIban.c_str());
+  if (!_localBic.empty())
+    AB_Transaction_SetRemoteBic(t, _localBic.c_str());
   if (!_localSuffix.empty())
     AB_Transaction_SetLocalSuffix(t, _localSuffix.c_str());
   if (!_localOwnerName.empty())
@@ -215,6 +229,8 @@ AB_TRANSACTION *Transaction::toBankingTransaction() {
     AB_Transaction_SetRemoteAccountNumber(t, _remoteAccountNumber.c_str());
   if (!_remoteIban.empty())
     AB_Transaction_SetRemoteIban(t, _remoteIban.c_str());
+  if (!_remoteBic.empty())
+    AB_Transaction_SetRemoteBic(t, _remoteBic.c_str());
   if (!_remoteName.empty()) {
     std::list<std::string>::iterator sit;
 
@@ -226,18 +242,8 @@ AB_TRANSACTION *Transaction::toBankingTransaction() {
     }
   }
 
-#if 0
-  if (!_categories.empty()) {
-    std::list<std::string>::iterator sit;
-
-    for (sit=_categories.begin();
-         sit!=_categories.end();
-         sit++) {
-      if (!(*sit).empty())
-        AB_Transaction_AddCategory(t, (*sit).c_str(), 0);
-    }
-  }
-#endif
+  if (!_category.empty())
+    AB_Transaction_AddCategory(t, _category.c_str(), 1);
 
   if (_valutaDate)
     AB_Transaction_SetValutaDate(t, _valutaDate);
@@ -287,6 +293,8 @@ Transaction::Transaction(const Transaction &t)
   _localCountry=t._localCountry;
   _localBankCode=t._localBankCode;
   _localAccountNumber=t._localAccountNumber;
+  _localIban=t._localIban;
+  _localBic=t._localBic;
   _localSuffix=t._localSuffix;
   _localOwnerName=t._localOwnerName;
 
@@ -297,6 +305,7 @@ Transaction::Transaction(const Transaction &t)
   _remoteAccountNumber=t._remoteAccountNumber;
   _remoteSuffix=t._remoteSuffix;
   _remoteIban=t._remoteIban;
+  _remoteBic=t._remoteBic;
   _remoteName=t._remoteName;
   _category=t._category;
   _payee=t._payee;
@@ -415,6 +424,34 @@ void Transaction::setLocalSuffix(const std::string &s){
 
 
 
+const std::string &Transaction::getLocalIban() const{
+  return _localIban;
+}
+
+
+
+void Transaction::setLocalIban(const std::string &s){
+  _localIban=s;
+  _hash.erase();
+  _modified=true;
+}
+
+
+
+const std::string &Transaction::getLocalBic() const{
+  return _localBic;
+}
+
+
+
+void Transaction::setLocalBic(const std::string &s){
+  _localBic=s;
+  _hash.erase();
+  _modified=true;
+}
+
+
+
 const std::string &Transaction::getLocalName() const{
   return _localOwnerName;
 }
@@ -499,6 +536,20 @@ void Transaction::setRemoteIban(const std::string &s){
 
 
 
+const std::string &Transaction::getRemoteBic() const{
+  return _remoteBic;
+}
+
+
+
+void Transaction::setRemoteBic(const std::string &s){
+  _remoteBic=s;
+  _hash.erase();
+  _modified=true;
+}
+
+
+
 const std::string &Transaction::getRemoteAccountNumber() const{
   return _remoteAccountNumber;
 }
@@ -947,6 +998,12 @@ bool Transaction::toDb(GWEN_DB_NODE *db) const {
   if (!_localSuffix.empty())
     GWEN_DB_SetCharValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS,
                          "localSuffix", _localSuffix.c_str());
+  if (!_localIban.empty())
+    GWEN_DB_SetCharValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS,
+                         "localIban", _localIban.c_str());
+  if (!_localBic.empty())
+    GWEN_DB_SetCharValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS,
+                         "localBic", _localBic.c_str());
 
   if (!_remoteCountry.empty())
     GWEN_DB_SetCharValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS,
@@ -960,6 +1017,9 @@ bool Transaction::toDb(GWEN_DB_NODE *db) const {
   if (!_remoteIban.empty())
     GWEN_DB_SetCharValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS,
                          "remoteIban", _remoteIban.c_str());
+  if (!_remoteBic.empty())
+    GWEN_DB_SetCharValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS,
+                         "remoteBic", _remoteBic.c_str());
   if (!_remoteBankName.empty())
     GWEN_DB_SetCharValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS,
                          "remoteBankName", _remoteBankName.c_str());
@@ -1063,12 +1123,15 @@ bool Transaction::fromDb(GWEN_DB_NODE *db) {
   _localCountry=GWEN_DB_GetCharValue(db, "localCountry", 0, "");
   _localBankCode=GWEN_DB_GetCharValue(db, "localBankCode", 0, "");
   _localAccountNumber=GWEN_DB_GetCharValue(db, "localAccountNumber", 0, "");
+  _localIban=GWEN_DB_GetCharValue(db, "localIban", 0, "");
+  _localBic=GWEN_DB_GetCharValue(db, "localBic", 0, "");
 
   _remoteCountry=GWEN_DB_GetCharValue(db, "remoteCountry", 0, "");
   _remoteBankCode=GWEN_DB_GetCharValue(db, "remoteBankCode", 0, "");
   _remoteBankName=GWEN_DB_GetCharValue(db, "remoteBankName", 0, "");
   _remoteBankLocation=GWEN_DB_GetCharValue(db, "remoteBankLocation", 0, "");
   _remoteIban=GWEN_DB_GetCharValue(db, "remoteIban", 0, "");
+  _remoteBic=GWEN_DB_GetCharValue(db, "remoteBic", 0, "");
   _remoteAccountNumber=GWEN_DB_GetCharValue(db, "remoteAccountNumber",
                                             0, "");
   _category=GWEN_DB_GetCharValue(db, "category", 0, "");
