@@ -1,7 +1,7 @@
 /***************************************************************************
  $RCSfile$
                              -------------------
-    cvs         : $Id: stoview.cpp 410 2007-09-04 07:07:26Z martin $
+    cvs         : $Id$
     begin       : Mon Mar 01 2004
     copyright   : (C) 2004 by Martin Preuss
     email       : martin@libchipcard.de
@@ -76,42 +76,48 @@ StandingOrderView::~StandingOrderView(){
 
 
 bool StandingOrderView::init(){
-  GWEN_DB_NODE *db;
+  GWEN_DB_NODE *dbConfig=NULL;
+  int rv;
+  std::string path;
   std::string s;
   int i, j;
 
-  db=_app->getAppData();
-  if (db) {
-    db=GWEN_DB_GetGroup(db, GWEN_PATH_FLAGS_NAMEMUSTEXIST,
-                        "gui/views/transferview");
-    if (db) {
-      int w;
-      int h;
-      const char *p;
+  path="gui/views/stoview";
+  rv=_app->loadAppSubConfig(path.c_str(),
+			    &dbConfig,
+			    0);
+  if (rv<0) {
+    DBG_INFO(0, "here (%d)", rv);
+  }
+  else {
+    int w;
+    int h;
+    const char *p;
 
-      /* found settings */
-      p=GWEN_DB_GetCharValue(db, "sortOrder", 0, "ascending");
-      if (p) {
-        if (strcasecmp(p, "ascending")==0)
-          _xaList->setSortOrder(Qt::Ascending);
-        else
-          if (strcasecmp(p, "descending")==0)
-            _xaList->setSortOrder(Qt::Descending);
-      }
-      i=GWEN_DB_GetIntValue(db, "sortColumn", 0, -1);
-      if (i!=-1)
-        _xaList->setSortColumn(i);
-      w=GWEN_DB_GetIntValue(db, "width", 0, 640);
-      h=GWEN_DB_GetIntValue(db, "height", 0, 480);
-      resize(w, h);
-      for (i=0; i<_xaList->columns(); i++) {
-        _xaList->setColumnWidthMode(i, QListView::Manual);
-        j=GWEN_DB_GetIntValue(db, "columns", i, -1);
-        if (j!=-1)
-          _xaList->setColumnWidth(i, j);
-      } /* for */
-    } // if transaction view setting
+    /* found settings */
+    p=GWEN_DB_GetCharValue(dbConfig, "sortOrder", 0, "ascending");
+    if (p) {
+      if (strcasecmp(p, "ascending")==0)
+	_xaList->setSortOrder(Qt::Ascending);
+      else
+	if (strcasecmp(p, "descending")==0)
+	  _xaList->setSortOrder(Qt::Descending);
+    }
+    i=GWEN_DB_GetIntValue(dbConfig, "sortColumn", 0, -1);
+    if (i!=-1)
+      _xaList->setSortColumn(i);
+    w=GWEN_DB_GetIntValue(dbConfig, "width", 0, 640);
+    h=GWEN_DB_GetIntValue(dbConfig, "height", 0, 480);
+    resize(w, h);
+    for (i=0; i<_xaList->columns(); i++) {
+      _xaList->setColumnWidthMode(i, QListView::Manual);
+      j=GWEN_DB_GetIntValue(dbConfig, "columns", i, -1);
+      if (j!=-1)
+	_xaList->setColumnWidth(i, j);
+    } /* for */
+    GWEN_DB_Group_free(dbConfig);
   } // if application settings
+
   slotUpdated();
 
   return true;
@@ -120,33 +126,30 @@ bool StandingOrderView::init(){
 
 
 bool StandingOrderView::fini(){
-  GWEN_DB_NODE *db;
+  GWEN_DB_NODE *dbConfig;
+  std::string path;
+  int rv;
   int i, j;
   std::string s;
 
-  db=_app->getAppData();
-  assert(db);
-  db=GWEN_DB_GetGroup(db,
-                      GWEN_DB_FLAGS_OVERWRITE_GROUPS,
-                      "gui/views/transferview");
-  assert(db);
-  db=GWEN_DB_GetGroup(db, GWEN_DB_FLAGS_OVERWRITE_GROUPS,
-                      s.c_str());
-  assert(db);
-  GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_DEFAULT,
+  path="gui/views/stoview";
+  dbConfig=GWEN_DB_Group_new("config");
+  assert(dbConfig);
+
+  GWEN_DB_SetIntValue(dbConfig, GWEN_DB_FLAGS_DEFAULT,
                       "width", width());
-  GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_DEFAULT,
+  GWEN_DB_SetIntValue(dbConfig, GWEN_DB_FLAGS_DEFAULT,
                       "height", height());
 
-  GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_DEFAULT,
+  GWEN_DB_SetIntValue(dbConfig, GWEN_DB_FLAGS_DEFAULT,
                       "sortColumn", _xaList->sortColumn());
   switch(_xaList->sortOrder()) {
   case Qt::Ascending:
-    GWEN_DB_SetCharValue(db, GWEN_DB_FLAGS_DEFAULT,
+    GWEN_DB_SetCharValue(dbConfig, GWEN_DB_FLAGS_DEFAULT,
                          "sortOrder", "ascending");
     break;
   case Qt::Descending:
-    GWEN_DB_SetCharValue(db, GWEN_DB_FLAGS_DEFAULT,
+    GWEN_DB_SetCharValue(dbConfig, GWEN_DB_FLAGS_DEFAULT,
                          "sortOrder", "descending");
     break;
   default:
@@ -155,10 +158,19 @@ bool StandingOrderView::fini(){
 
   for (i=0; i<_xaList->columns(); i++) {
     j=_xaList->columnWidth(i);
-    GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_DEFAULT,
+    GWEN_DB_SetIntValue(dbConfig, GWEN_DB_FLAGS_DEFAULT,
                         "columns", j);
   } /* for */
 
+  rv=_app->saveAppSubConfig(path.c_str(),
+			    dbConfig,
+			    0);
+  GWEN_DB_Group_free(dbConfig);
+  if (rv<0) {
+    DBG_INFO(0, "here (%d)", rv);
+    return false;
+  }
+
   return true;
 }
 
